unit rk_; //   -

interface

type
	DVector = array of extended;	//  
	RungeKutt = class				// ------------- RungeKutt -----------------
    public
      N: integer; 					//  
      order: integer;	 			//  
      a: array of DVector;			// 
      c: array of extended;			//  c_i

      dim: integer;					//  
      xt:DVector;					//  
      k: array of Dvector;			//  k_i
    public
      t :         extended;			// 
      //x0:DVector;					//  
      constructor Create;
      Destructor  Done;
      procedure   Clear;


      function    read1( var f: text ):boolean; //  .    
      function    read(in_name: string):boolean;//  . 
      function    read0(in_name: string; var inp: text):boolean;//  .     
      function    set44(u,v: extended):boolean; //  (4,4)-   (u,v)
      procedure	  set44_std();  		       	//   (4,4)-
      procedure	  set44_3_8();  		       	//  (4,4)-:  3/8
      function	  set56(c2,c3,c4,c5,b2,a43: extended):boolean; //  (5,6)-   (c2...)
      function	  set56n(c_2,c_3,c_5,c_6,h: extended):boolean; //  (5,6)-   (c2...)      
      function	  max_coeff:extended; 			//   

      procedure	  initBuf(dim_: integer);								//      dim
      procedure	  f(t: extended; var x,y: DVector); virtual;	// (t,x) -> y
      function    dst_vect( var y1, y2: DVector): extended; virtual; 	//   
      function    err( var x: DVector): extended; virtual; 				//    (    -  0)
	  procedure   save( var y: DVector; fname: string); virtual; 		//   
	  procedure   save0( fname: string); virtual; 		//   
	  procedure   app ( var y: DVector; fname: string); virtual; 		//   
    end;

procedure cop_vect( var dst, src: DVector); // dst := src
procedure add_vect( var y1, y2: DVector; k: extended); // y1 += y2*k
procedure sav_vect( var y: DVector; fname: string); //   
procedure app_vect( var y: DVector; fname: string); //   

implementation

uses SysUtils, Math, common;
//==============================================================================
// -----------------------------------------------------------------------
constructor RungeKutt.Create;
begin
	self.Clear;
end;
Destructor  RungeKutt.Done;
begin
	self.Clear;
end;
//-------------------------------------------------------------------------
procedure   RungeKutt.Clear;
begin
	N:=0;
    order:=0;
    dim:=0;
    t:=0;
    a:=nil;
    c:=nil;
    xt:=nil;
    k:=nil;
end;
//-------------------------------------------------------------------------
function RungeKutt.read1( var f: text ):boolean; //  .    
var s      : string;
    i,j    : integer;
    ci	   : extended;
begin
    ReadLn( f, s );
    ReadLn( f, N ); 	  // 
  if (N<2) or (N>20)  then ErrProc( 'Illegal number of steps');
    ReadLn( f, order );       // 
	if (order>N) or (order<2) then ErrProc( 'Illegal p in read_RK');
    ReadLn( f, s );       // 
    SetLength( a, N+2, N+2 );
    SetLength( c, N+2 );
    for i:=0 to high(a) do
    for j:=0 to high(a[0]) do a[i,j]:=0;

    for i:= 2 to high(a) do
    for j:= 1 to i-1 do ReadLn( f, a[i,j] );
    result := true;

	while true do begin
	  	ReadLn( f, s );
        s := trim(s);
        if Length(s)>1 then break;
    end;
  	if copy(s, 1, 3) <> '---' then ErrProc( 'Illegal last line in input file');

	for i:=1 to N do begin
		ci:=0; for j:=1 to i-1 do ci:=ci+a[i,j];
        c[i]:=ci;
    end;
    c[N+1]:=1;
end;
//-------------------------------------------------------------------------
function    RungeKutt.read( in_name: string ):boolean; 		//  . 
var inp    : text;
begin
	result := false;
	{$I-}
	Assign( inp, in_name );  Reset ( inp );
	{$I+}
	if IOResult <> 0 then Exit;		// ErrProc('File '+in_name+' not open');
	result := self.read1(inp);
    Close(inp);
end;
//-------------------------------------------------------------------------
function    RungeKutt.read0(in_name: string; var inp: text):boolean;//  .     
begin
	result := false;
	{$I-}
	Assign( inp, in_name );  Reset ( inp );
	{$I+}
	if IOResult <> 0 then Exit;		// ErrProc('File '+in_name+' not open');
	result := self.read1(inp);
end;
// -----------------------------------------------------------------------------
// c2 = u,
// c3 = v.
function RungeKutt.set44(u,v: extended):boolean; //  (4,4)-   (u,v)
var q1: extended;
    i,j: integer;
    //f: text;
begin
    result := false;
    if (u=0) or (u=0.5) or (u=1) or (v=0) or (v=1) or (v=u) then
        //ErrProc('RungeKutt.set44: Illegal u='+N2Str(u,10)+', v='+N2str(v,10));
        Exit;
    q1 := 3-4*u-4*v+6*u*v;
    if (q1=0) then
        //ErrProc('RungeKutt.set44: Illegal q1; u='+N2Str(u,10)+', v='+N2str(v,10));
        Exit;

    N := 4; order:=4;
    SetLength( a, N+2, N+2 );
    SetLength( c, N+2 );
    for i:=0 to high(a) do
    for j:=0 to high(a[0]) do a[i,j]:=0;

    c[0]:=0; c[1]:=0;
    c[2]:=u; c[3]:=v;
    c[4]:=1; c[5]:=1;

    a[2,1] := u;

    //a[3,1] := v*(4*u*u-3*u+v)/(2*u*(2*u-1));
    a[3,2] := v*(v-u)/(2*u*(1-2*u));
    a[3,1] := v - a[3,2];

    a[4,2] := (u-4*v*v+5*v-2)*(u-1)/(2*u*(u-v)*q1);
    a[4,3] := (u-1)*(v-1)*(2*u-1)/(v*(u-v)*q1);
    a[4,1] := 1 - a[4,2] - a[4,3];

    //a51 := (6*c2*c3 - 2*c2 - 2*c3 + 1)/(12*c2*c3):
    a[5,2] := (2*v - 1)/(12*(u - v)*(u - 1)*u);
    a[5,3] := ( -(2*u - 1))/(12*(u - v)*(v - 1)*v);
    a[5,4] := (6*u*v - 4*u - 4*v + 3)/(12*(u - 1)*(v - 1));
    a[5,1] := 1 - a[5,2] - a[5,3] - a[5,4];

    //hOpen('t:\rk.txt', f);
    //WriteLn( f, u:10:7,' ', v:10:7,' '
    //	, a[2,1]:10:7,' '
    //	, a[3,1]:10:7,' ', a[3,2]:10:7,' '
    //    , a[4,1]:10:7,' ', a[4,2]:10:7,' ', a[4,3]:10:7,' '
    //    , a[5,1]:10:7,' ', a[5,2]:10:7,' ', a[5,3]:10:7,' ', a[5,4]:10:7,' '
    //    );
    //CloseFile(f);
    result := true;
end;
// -----------------------------------------------------------------------------
procedure  RungeKutt.set44_std();  		       	//   (4,4)-
var i,j: integer;
begin
    N := 4; order:=4;
    SetLength( a, N+2, N+2 );
    SetLength( c, N+2 );
    for i:=0 to high(a) do
    for j:=0 to high(a[0]) do a[i,j]:=0;

    c[0]:=0;   c[1]:=0;
    c[2]:=0.5; c[3]:=0.5;
    c[4]:=1;   c[5]:=1;

    a[2,1] := 0.5;

    a[3,1] := 0;
    a[3,2] := 0.5;

    a[4,1] := 0;
    a[4,2] := 0;
    a[4,3] := 1;

    a[5,1] := 1./6;
    a[5,2] := 1./3;
    a[5,3] := 1./3;
    a[5,4] := 1./6;
end;
// -----------------------------------------------------------------------------
procedure  RungeKutt.set44_3_8();  		       	//  (4,4)-:  3/8
var i,j: integer;
begin
    N := 4; order:=4;
    SetLength( a, N+2, N+2 );
    SetLength( c, N+2 );
    for i:=0 to high(a) do
    for j:=0 to high(a[0]) do a[i,j]:=0;

    c[0]:=0;   c[1]:=0;
    c[2]:=1./3;c[3]:=2./3;
    c[4]:=1;   c[5]:=1;

    a[2,1] := 1./3;

    a[3,1] := -1./3;
    a[3,2] := 1;

    a[4,1] := 1;
    a[4,2] := -1;
    a[4,3] := 1;

    a[5,1] := 0.125;
    a[5,2] := 0.375;
    a[5,3] := 0.375;
    a[5,4] := 0.125;
end;
// -----------------------------------------------------------------------------
//  b3,b4,b5,b6   :
// b3*c3   + b4*c4    + b5*c5    + b6*c6    = z1;
// b3*c3^2 + b4*c4^2  + b5*c5^2  + b6*c6^2  = z2;
// b3*c3^3 + b4*c4^3  + b5*c5^3  + b6*c6^3  = z3;
// b3*c3^4 + b4*c4^4  + b5*c5^4  + b6*c6^4  = z4;
//   ,  false
function VadDerMond_4( c3,c4,c5,c6, z1,z2,z3,z4: extended; var b3,b4,b5,b6: extended):boolean;
var den:extended;
begin
	//b3 := (-c5*z2*c4+c4*c6*c5*z1+z3*c4-c4*c6*z2+z3*c5-c5*c6*z2-z4+c6*z3)/(c6-c3)/(c3-c5)/(c3-c4)/c3;
    //b4 := (-c3*c5*z2+c3*c6*c5*z1+c3*z3-c3*c6*z2+z3*c5-c5*c6*z2-z4+c6*z3)/(c4-c6)/(c4-c5)/(c3-c4)/c4;
    //b5 := (-z2*c3*c4+c3*c4*z1*c6+c3*z3-c3*c6*z2+z3*c4-c4*c6*z2-z4+c6*z3)/(c6-c5)/(c4-c5)/(c3-c5)/c5;
    //b6 := (-c5*z2*c4-z2*c3*c4+z3*c4+c3*c5*z1*c4+c3*z3+z3*c5-c3*c5*z2-z4)/c6/(-c6+c5)/(c4-c6)/(-c6+c3);
    result := false;

	den:= (c6-c3)*(c3-c5)*(c3-c4)*c3;
	if den=0 then Exit;					//  
	b3 := (-c5*z2*c4+c4*c6*c5*z1+z3*c4-c4*c6*z2+z3*c5-c5*c6*z2-z4+c6*z3)/den;

	den:= (c4-c6)*(c4-c5)*(c3-c4)*c4;
	if den=0 then Exit;					//  
    b4 := (-c3*c5*z2+c3*c6*c5*z1+c3*z3-c3*c6*z2+z3*c5-c5*c6*z2-z4+c6*z3)/den;

	den:= (c6-c5)*(c4-c5)*(c3-c5)*c5;
	if den=0 then Exit;					//  
    b5 := (-z2*c3*c4+c3*c4*z1*c6+c3*z3-c3*c6*z2+z3*c4-c4*c6*z2-z4+c6*z3)/den;

    den:= c6*(c6-c5)*(c4-c6)*(c6-c3);
	if den=0 then Exit;					//  
    b6 := (-c5*z2*c4-z2*c3*c4+z3*c4+c3*c5*z1*c4+c3*z3+z3*c5-c3*c5*z2-z4)/den;

    //    ,   :
    result := true;
end;
// -----------------------------------------------------------------------------
//  d3,d4,d5,d6   :
// b3*d3      + b4*d4      + b5*d5      + b6*d6   = 1/6;
// b3*c3*d3   + b4*c4*d4   + b5*c5*d5   + b6*d6   = 1/8;
// b3*c3^2*d3 + b4*c4^2*d4 + b5*c5^2*d5 + b6*d6   = 1/10;
// b3*d3^2    + b4*d4^2    + b5*d5^2    + b6*d6^2 = 1/20;
//   ,  false
//   ,   ,    knd:
//       knd<0,       d6,  -  
function System_d( c3,c4,c5, b3,b4,b5,b6: extended; knd: integer; var d3,d4,d5,d6: extended):integer;
var den:extended;
	u3,u4,u5, v3,v4,v5: extended;
    A, B, C, D: extended;
begin
	result := 0;
	//       di  d6:
    // d3 = u3 + v3*d6;
    // d4 = u4 + v4*d6;
    // d5 = u5 + v5*d6;
    den := (c4-c3)*(c5-c3)*b3;
	if den=0 then Exit;					//  
    u3  := (c4*c5/6 - c4/8 - c5/8 + 1./10)/den;
    v3  := -b6*(c5-1)*(c4-1)/den;

    den := (c4-c3)*(c4-c5)*b4;
	if den=0 then Exit;					//  
    u4  := (c3*c5/6 - c5/8 - c3/8  + 1./10)/den;
    v4  := -b6*(c5-1)*(c3-1)/den;

    den := (c5-c3)*(c5-c4)*b5;
	if den=0 then Exit;					//  
    u5  := (c3*c4/6 - c4/8 - c3/8  + 1./10)/den;
    v5  := -b6*(c3-1)*(c4-1)/den;

    // : System_d( 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 1, d3,d4,d5,d6);
    //   
    // d3 := -49.*d6+5.937500001
    // d4 := 88.2*d6-11.0
    // d5 := -42.*d6+5.486111110

    //d6 := 1;
    //d3 := u3 + v3*d6;		// d3 = -43.06250000
    //d4 := u4 + v4*d6; 	// d4 =  77.20000000
    //d5 := u5 + v5*d6;		// d5 = -36.51388889

	//u3 := b3*d3      + b4*d4      + b5*d5      + b6*d6; //   = 1/6;
	//u4 := b3*c3*d3   + b4*c4*d4   + b5*c5*d5   + b6*d6; //   = 1/8;
	//u5 := b3*c3*c3*d3+ b4*c4*c4*d4+ b5*c5*c5*d5+ b6*d6; //   = 1/10;

    //WriteLn( u3,u4,u5 );

	// :
    // d3 = u3 + v3*d6;
    // d4 = u4 + v4*d6;
    // d5 = u5 + v5*d6;
    //   

    //     :
	// b3*d3^2    + b4*d4^2    + b5*d5^2    + b6*d6^2 = 1/20;
    // 

    // d6^2*(b3*v3^2 + b4*v4^2 + b5*v5^2 + b6 ) +
    // d6*2*(b3*u3*v3 + b4*u4*v4 + b5*u5*v5) +
    //      (b3*u3^2 + b4*u4^2 + b5*u5^2 - 1/20)

    // A*y^2 + B*y + C=0
    A := b3*v3*v3 + b4*v4*v4 + b5*v5*v5 + b6;
    B := 2*(b3*u3*v3 + b4*u4*v4 + b5*u5*v5);
    C := b3*u3*u3 + b4*u4*u4 + b5*u5*u5 - 1./20;
    if A=0 then Exit;			//   

	// 
    D := B*B - 4*A*C;
    if abs(D)<1e-15 then begin D:=0; result:=1; end;
    if D<0 then Exit;			//  

    D := sqrt(D);
    if knd<0 then D := -D;

    d6 := (-B+D)/(2*A);

    d3 := u3 + v3*d6;
    d4 := u4 + v4*d6;
    d5 := u5 + v5*d6;

    //  :
	//u3 := b3*d3      + b4*d4      + b5*d5      + b6*d6; //   = 1/6;
	//u4 := b3*c3*d3   + b4*c4*d4   + b5*c5*d5   + b6*d6; //   = 1/8;
	//u5 := b3*c3*c3*d3+ b4*c4*c4*d4+ b5*c5*c5*d5+ b6*d6; //   = 1/10;
	//v3 := b3*d3*d3   + b4*d4*d4   + b5*d5*d5   + b6*d6*d6;// = 1/20;
    //WriteLn( u3,u4,u5,v3);

    result :=2;
end;

// Maple-   :
//
//> restart: with(linalg):
//> c6:=1: z1:=1/6: z2:=1/8: z3:=1/10: z4:=1/20:
//#
//> #c3:=0.1: c4:=0.2: c5:=0.3: b3:=0.4: b4:=0.5: b5:=0.6: b6:=0.7:
//#
//> eq1 := b3*d3      + b4*d4      + b5*d5      + b6*d6      - z1;
//> eq2 := b3*c3*d3   + b4*c4*d4   + b5*c5*d5   + b6*c6*d6   - z2;
//> eq3 := b3*c3^2*d3 + b4*c4^2*d4 + b5*c5^2*d5 + b6*c6^2*d6 - z3;
//> eq4 := b3*d3^2    + b4*d4^2    + b5*d5^2    + b6*d6^2    - z4;
//> sol := solve({eq1,eq2,eq3},{d3,d4,d5});
//> assign(sol);
//> d3:=factor(d3);
//> den := (c4-c3)*(c3-c5)*b3;
//> collect(d3*den,d6,factor);
//> den := (c4-c3)*(c4-c5)*b4;
//> d4:=factor(d4);
//> collect(d4*den,d6,factor);
//> d5:=factor(d5);
//> den := (c3-c5)*(c4-c5)*b5;
//> collect(d5*den,d6,factor);
//> #eq4 := factor(eq4);
//    -  5 

function RungeKutt.set56n(c_2,c_3,c_5,c_6,h: extended):boolean;
var
p,c_4,p1,p2: extended;
i,j: integer;
begin
 result := false;
 N := 6; order:=5;
 SetLength( a, N+2, N+2 );
 SetLength( c, N+2 );
 for i:=0 to high(a) do
  for j:=0 to high(a[0]) do a[i,j]:=0;
 for i:=2 to high(a) do
  for j:=1 to i-1 do a[i,j]:=777;
 p1:=5*c_3*c_3-4*c_3+1;
 p2:=10*c_3*c_3-8*c_3+1;
 p:=sqr(p1)*p1;
 if (p1=0)or(c_2=0)or(c_3=0)or(c_5=0)or(c_6=0)
 or(c_3-c_5=0)or(c_3-c_6=0)or(c_5-c_6=0)
 or(20*c_5*c_3-15*c_3-10*c_5+8=0)or((3-12*c_3+10*sqr(c_3))=0)or(10*sqr(c_3)-8*c_3+1=0)
 or(10*c_5*c_3*c_3-8*c_5*c_3+2*c_5-c_3=0)or(10*c_6*c_3*c_3-8*c_6*c_3+2*c_6-c_3=0) then Exit;

  c_4:=(1/2)*c_3/(5*sqr(c_3)-4*c_3+1);
  a[2,1]:=c_2;

  a[3,2]:=sqr(c_3)/(2*c_2);
  a[3,1]:=c_3-a[3,2];

  a[4,2]:=(1/8)*(15*sqr(c_3)-12*c_3+2)*sqr(c_3)/(p*c_2);
  a[4,3]:=-(1/8)*(10*sqr(c_3)-8*c_3+1)*c_3/p;
  a[4,1]:=c_4-a[4,2]-a[4,3];

  a[5,2]:=(1/2)*(c_3*(3-12*c_5+10*sqr(c_5))*c_5)/(c_2*(3-12*c_3+10*sqr(c_3)));
  a[5,3]:=(1/2)*(80*c_5*sqr(c_3)*c_3-94*c_5*sqr(c_3)+32*c_5*c_3-4*c_5-20*c_3*sqr(c_3)-c_3+16*sqr(c_3))*(c_3-c_5)*c_5/(sqr(c_3)*(10*sqr(c_3)-8*c_3+1)*(3-12*c_3+10*sqr(c_3)));
  a[5,4]:=2*sqr(5*sqr(c_3)-4*c_3+1)*(10*c_5*sqr(c_3)-8*c_5*c_3+2*c_5-c_3)*(c_3-c_5)*c_5/(sqr(c_3)*(10*sqr(c_3)-8*c_3+1)*(3-12*c_3+10*sqr(c_3)));
  a[5,1]:=c_5-a[5,2]-a[5,3]-a[5,4];

  a[6,2]:=(1/2)*c_3*(-12*c_6+10*sqr(c_6)+3)*c_6/((3-12*c_3+10*sqr(c_3))*c_2);
  a[6,3]:=(1/2)*c_6*(c_3-c_6)*(-1200*c_6*sqr(c_3)*sqr(c_3)*c_3+1950*c_6*sqr(c_3)*sqr(c_3)-1072*c_6*sqr(c_3)*c_3+1896*c_6*sqr(c_3)*c_5-528*c_6*c_3*c_5-1064*sqr(c_6)*sqr(c_3)+252*sqr(c_6)*c_3-40*c_6*sqr(c_5)+56*c_6*c_5+920*c_6*c_3*c_3*c_3*c_3*c_5+300*c_3*c_3*c_3*c_3*c_3-20*c_6*c_3-24*sqr(c_6)-520*sqr(c_5)*c_3*c_3*c_3+400*sqr(c_5)*c_3*c_3*c_3*c_3-10*sqr(c_5)*c_3+180*sqr(c_5)*c_3*c_3+2260*sqr(c_6)*c_3*c_3*c_3-2400*sqr(c_6)*c_3*c_3*c_3*c_3-8*c_3*c_3-1580*c_6*sqr(c_5)*c_3*c_3+60*c_5*c_3*c_3*c_3+320*c_5*c_3*c_3*c_3*c_3-400*c_5*c_3*c_3*c_3*c_3*c_3-4*c_5*c_3-55*c_5*c_3*c_3+1000*c_6*c_6*c_3*c_3*c_3*c_3*c_3-1600*c_6*sqr(c_5)*c_3*c_3*c_3*c_3-400*c_3*c_3*c_3*c_3+238*c_6*c_3*c_3+143*c_3*c_3*c_3-2730*c_6*c_3*c_3*c_3*c_5+2680*c_3*c_3*c_3*c_6*sqr(c_5)+600*c_6*sqr(c_3)*sqr(c_3)*c_3*c_5+400*c_6*c_3*sqr(c_5))/(c_3*c_3*(c_3-c_5)*(3-12*c_3+10*sqr(c_3))*(10*sqr(c_3)-8*c_3+1)*(20*c_5*c_3-15*c_3-10*c_5+8));
  a[6,4]:=2*(10*c_6*c_3*c_3-8*c_6*c_3+2*c_6-c_3)*c_6*sqr(5*c_3*c_3-4*c_3+1)*(c_3-c_6)*(100*c_6*c_3*c_3*c_3-250*c_5*c_3*c_3*c_3+200*c_5*c_5*c_3*c_3*c_3-260*c_5*c_5*c_3*c_3-160*c_6*c_3*c_3+15*c_3*c_3+340*c_5*c_3*c_3+78*c_6*c_3-162*c_5*c_3+120*c_5*c_5*c_3-8*c_3-20*c_5*c_5-12*c_6+28*c_5)/(c_3*c_3*(3-12*c_3+10*c_3*c_3)*(20*c_5*c_3-15*c_3-10*c_5+8)*(10*c_5*c_3*c_3-8*c_5*c_3+2*c_5-c_3)*(10*c_3*c_3-8*c_3+1));
  a[6,5]:=(-2+5*c_3)*(10*c_6*c_3*c_3-8*c_6*c_3+2*c_6-c_3)*(c_3-c_6)*(-c_6+c_5)*c_6/(c_5*(20*c_5*c_3-15*c_3-10*c_5+8)*(10*c_5*c_3*c_3-8*c_5*c_3+2*c_5-c_3)*(c_3-c_5));
  a[6,1]:=c_6-a[6,2]-a[6,3]-a[6,4]-a[6,5];

  a[7,2] := 0;
  a[7,3] := (-150*c_5*c_3*c_3+200*c_6*c_3*c_3*c_5-150*c_6*c_3*c_3+120*c_3*c_3-190*c_6*c_3*c_5+140*c_5*c_3-111*c_3+140*c_3*c_6-30*c_5+40*c_6*c_5+24-30*c_6)/(60*c_3*c_3*p2*(c_3-c_5)*(c_3-c_6));
  a[7,4] := 4*p1*p1*p1*p1*(-20*c_5*c_3-20*c_3*c_6+30*c_6*c_3*c_5+15*c_3-12+15*c_5-20*c_5*c_6+15*c_6)/(15*c_3*c_3*p2*(10*c_5*c_3*c_3-8*c_5*c_3+2*c_5-c_3)*(10*c_6*c_3*c_3-8*c_6*c_3+2*c_6-c_3));
  a[7,5] := -(3-12*c_3+10*c_3*c_3)*(20*c_6*c_3-15*c_3-10*c_6+8)/(60*c_5*(10*c_5*c_3*c_3-8*c_5*c_3+2*c_5-c_3)*(c_3-c_5)*(c_5-c_6));
  a[7,6] := (3-12*c_3+10*c_3*c_3)*(20*c_5*c_3-15*c_3-10*c_5+8)/(60*c_6*(10*c_6*c_3*c_3-8*c_6*c_3+2*c_6-c_3)*(c_3-c_6)*(c_5-c_6));
  a[7,1] := 1-a[7,2]-a[7,3]-a[7,4]-a[7,5]-a[7,6];

  result := true;
end;
// -----------------------------------------------------------------------------
function RungeKutt.set56(c2,c3,c4,c5,b2,a43: extended):boolean; //  (5,6)-   (c2...)
var
	i,j: integer;
    b3,b4,b5,b6, d3,d4,d5,d6: extended;
    t1, t2: extended;
    knd: integer;
    //f: text;
begin
	result := false;
    //if (q1=0) then
    //    ErrProc('RungeKutt.set44: Illegal q1; u='+N2Str(u,10)+', v='+N2str(v,10));

    N := 6; order:=5;
    SetLength( a, N+2, N+2 );
    SetLength( c, N+2 );
    for i:=0 to high(a) do
    for j:=0 to high(a[0]) do a[i,j]:=0;

    for i:=2 to high(a) do
    for j:=1 to i-1 do a[i,j]:=777;


    c[0]:=0; c[1]:=0;
    c[2]:=c2; c[3]:=c3; c[4]:=c4; c[5]:=c5; a[7,2]:=b2; a[4,3]:=a43;
    c[6]:=1; c[7]:=1;

    a[2,1] := c2;
    a[4,3] := a43;
    a[7,2] := b2;

    if not VadDerMond_4( c3, c4, c5, 1,
    	1./2 - b2*c2,
        1./3 - b2*c2*c2,
        1./4 - b2*c2*c2*c2,
        1./5 - b2*c2*c2*c2*c2,
        b3,b4,b5,b6 ) then Exit;
	knd := -1; //  
    if System_d( c3,c4,c5, b3,b4,b5,b6, knd, d3,d4,d5,d6)<1 then Exit;
    a[7,3] := b3;
    a[7,4] := b4;
    a[7,5] := b5;
    a[7,6] := b6;
    a[7,1] := 1-b2-b3-b4-b5-b6;

    if c2=0 then Exit; 		//  
    a[3,2] :=  d3/c2;
    a[3,1] := c3-a[3,2];
    a[4,2] := (d4-a43*c3)/c2;
    a[4,1] := c4 - a[4,2] - a43;

    //  a[6,5]:
    if b6=0 then Exit;
    a[6,5] := b5*(1-c5)/b6;	//  112/15 = 7.46666666666666667

    //   a[5,3]:
    t2 := b6*a[6,5]*(c3*c3*d4-c2*c3*d4-d3*c4*c4+c2*d3*c4); //  -7/675=-0.01037037037037037
    if t2=0 then Exit;
	t1:=-2*d4+c4*(c4-c2)
    	+120*(
         c2*c3*c4*d4*b4*a43
        -c2*c4*c4*b4*d3*a43
        +c2*d4*d5*b6*a[6,5]
        +c3*c3*b4*d4*a43
        -c3*c3*c4*b4*d4*a43
        +c2*c4*b4*d3*a43
        -c2*c3*b3*d3*d4
        +c2*b3*d3*d4
        +c2*b4*d4*d4
        -c2*c4*b4*d4*d4
        -c2*c3*b4*d4*a43
        -c4*c4*b4*d3*a43
        +c4*c4*c4*b4*d3*a43
        ); // t1 = 5/9 = 0.55555555555555556
    a[5,3] := -t1/(t2*120);  // 25/56 = 0.44642857142857143 in example

	t2 := 120*b6*a[6,5]*d4;
    if t2=0 then Exit;
    t1 := 1
         -120*b4*a43*d3
         +120*b4*a43*d3*c4
         -120*b6*a[6,5]*a[5,3]*d3;
    a[5,4] := t1/t2;                     // 15/448 = 0.03348214285714286
    a[5,2] := (d5-a[5,3]*c3-a[5,4]*c4)/c2;
    a[5,1] := c5 - a[5,2] - a[5,3] - a[5,4];

	a[6,4] := -(b5*a[5,4]-b4+b4*c4)/b6;  // -10
    if b6=0 then Exit;

    a[6,3] := -(b4*a[4,3]+b5*a[5,3]-b3+b3*c3)/b6; // 5/3
    a[6,2] := (d6-a[6,3]*c3-a[6,4]*c4-a[6,5]*c5)/c2;
    a[6,1] := 1 - a[6,2] - a[6,3] - a[6,4] - a[6,5];

    //hOpen('t:\rk.txt', f);
    //WriteLn( f, 'rk56 SKH '
    //	, a[2,1]:10:7,' '
    //	, a[3,1]:10:7,' ', a[3,2]:10:7,' '
    //    , a[4,1]:10:7,' ', a[4,2]:10:7,' ', a[4,3]:10:7,' '
    //    , a[5,1]:10:7,' ', a[5,2]:10:7,' ', a[5,3]:10:7,' ', a[5,4]:10:7,' '
    //    , a[6,1]:10:7,' ', a[6,2]:10:7,' ', a[6,3]:10:7,' ', a[6,4]:10:7,' ', a[6,5]:10:7,' '
    //    , a[7,1]:10:7,' ', a[7,2]:10:7,' ', a[7,3]:10:7,' ', a[7,4]:10:7,' ', a[7,5]:10:7,' ', a[7,6]:10:7,' '
    //    );
    //WriteLn( f, 'rk56 SKH');
    //WriteLn( f, a[2,1]:10:7);
	//WriteLn( f, a[3,1]:10:7,' ', a[3,2]:10:7);
	//WriteLn( f, a[4,1]:10:7,' ', a[4,2]:10:7,' ', a[4,3]:10:7);
	//WriteLn( f, a[5,1]:10:7,' ', a[5,2]:10:7,' ', a[5,3]:10:7,' ', a[5,4]:10:7);
	//WriteLn( f, a[6,1]:10:7,' ', a[6,2]:10:7,' ', a[6,3]:10:7,' ', a[6,4]:10:7,' ', a[6,5]:10:7);
	//WriteLn( f, a[7,1]:10:7,' ', a[7,2]:10:7,' ', a[7,3]:10:7,' ', a[7,4]:10:7,' ', a[7,5]:10:7,' ', a[7,6]:10:7);
    //CloseFile(f);

{
 r.set56(1./3, 1./2, 2./3, 3./4, 9./25, 1./3);
  :
 d3 = 3/8;
 d4 = 1/2;
 d5 = 15/32;
 d6 = 0;


 [ 0       0       0      0      0      0      0]
 [                                              ]
 [1/3      0       0      0      0      0      0]
 [                                              ]
 [-5/8    9/8      0      0      0      0      0]
 [                                              ]
 [-2/3     1      1/3     0      0      0      0]
 [                                              ]
 [-179    75      25     15                     ]
 [----    ---     --     ---     0      0      0]
 [448     112     56     448                    ]
 [                                              ]
 [                              112             ]
 [7/6     7/10    5/3    -10    ---     0      0]
 [                              15              ]
 [                                              ]
 [                       -9     224             ]
 [1/9     9/25    2/5    --     ---    1/30    0]
 [                       10     225             ]



[.3333333333       0.           0.            0.           0.            0.      0.]
[-.6250000000  1.125000000      0.            0.           0.            0.      0.]
[-.6666666667      1.0      .3333333333       0.           0.            0.      0.]
[-.3995535714  .6696428571  .4464285714  .03348214286      0.            0.      0.]
[1.166666667   .7000000000  1.666666667     -10.0      7.466666667       0.      0.]
[.1111111111   .3600000000  .4000000000  -.9000000000  .9955555556  .03333333333 0.]

================================================================
 r.set56(1./3, 1./2, 2./3, 3./4, 9./25, 2./7);
  :

 [.3333333333        0.           0.            0.            0.            0.      0.]
 [-.6250000000  1.125000000       0.            0.            0.            0.      0.]
 [-.6904761905  1.071428571   .2857142857       0.            0.            0.      0.]
 [-.4282525510  .7557397959   .3890306122  .03348214286       0.            0.      0.]
 [1.380952381   .05714285714  2.095238095     -10.0       7.466666667       0.      0.]
 [.1111111111   .3600000000   .4000000000  -.9000000000   .9955555556  .03333333333 0.]

}
   result := true;
end; // function RungeKutt.set56(c2,c3,c4,c5,b2,a43: extended):boolean; //  (5,6)-   (c2...)
// -----------------------------------------------------------------------------
function RungeKutt.max_coeff:extended; 			//   
var i,j:integer;
    v: extended;
begin
	v := 0;
    for i:=2 to high(a) do
    for j:=1 to i-1 do
    	if abs(a[i,j])>v then v:=abs(a[i,j]);
    result := v;
end;
// -----------------------------------------------------------------------------
procedure  RungeKutt.initBuf(dim_: integer);		//      dim
begin
	dim := dim_;
    if (Length(k) <> (N+1)) or (Length(k[0]) <> dim) then SetLength( k, N+1, dim );
    //if Length(x0) <> dim then SetLength( x0,dim);
    if Length(xt) <> dim then SetLength( xt,dim);
end;
// ----------------------------------------------------------------------
procedure RungeKutt.f(t: extended; var x,y: DVector); // virtual; //abstract;		// (t,x) -> y
var i: integer;
begin
	for i:=0 to dim-1 do y[i]:=0;
end;
// -----------------------------------------------------------------------
function RungeKutt.dst_vect( var y1, y2: DVector): extended; //   
var i: integer;
	d: extended;
begin
	if (y1[0]>1e90) or (y2[0]>1e90) then begin result := 99; Exit; end;
	d:=0;
  	for i:=0 to high(y1) do d := d+ sqr(y1[i]-y2[i]);
    result := sqrt(d);
end;
// -----------------------------------------------------------------------
function RungeKutt.err( var x: DVector): extended; //    (    -  0)
begin
	result:=0;
	if abs(x[0])>1e90 then result := 1e99;
end;
// -----------------------------------------------------------------------
procedure RungeKutt.save( var y: DVector; fname: string); //   
var f: text;
	i,j: integer;
begin
  Assign( f, fname);
  Rewrite( f );

  for i:=1 to High(a) do begin
	  Write( f, N2Str(c[i],10), '; ' );
	  for j:=1 to High(a[i]) do
		  Write( f, N2Str(a[i][j],10), ' ' );
	  WriteLn( f);
  end;
  WriteLn( f, 'Vector:' );
  for i:=0 to High(y) do
  Write( f, N2Str(y[i],14), ' ' );
  WriteLn( f, '----------------------' );
  Close( f );
end;
// -----------------------------------------------------------------------
procedure RungeKutt.save0(fname: string); //   
var f: text;
	i,j: integer;
begin
  Assign( f, fname);
  Rewrite( f );

  for i:=1 to High(a) do begin
	  Write( f, N2Str(c[i],10), '; ' );
	  for j:=1 to High(a[i]) do
		  Write( f, N2Str(a[i][j],10), ' ' );
	  WriteLn( f);
  end;
  WriteLn( f, '----------------------' );
  Close( f );
end;
// -----------------------------------------------------------------------
procedure RungeKutt.app ( var y: DVector; fname: string); //   
var f: text;
	i,j: integer;
begin
  hOpen( fname, f);

  for i:=0 to High(a) do begin
	  Write( f, N2Str(c[i],10), '; ' );
	  for j:=0 to High(a[i]) do
		  Write( f, N2Str(a[i][j],10), ' ' );
	  WriteLn( f);
  end;
  WriteLn( f, 'Vector:' );
  for i:=0 to High(y) do
  Write( f, N2Str(y[i],14), ' ' );
  WriteLn( f, '----------------------' );
  Close( f );
end;
// -----------------------------------------------------------------------
procedure cop_vect( var dst,src: DVector); // y1 := y2
//var i: integer;
begin
	//if Length(dst)<>Length(src) then SetLength(dst, Length(src));
  	//for i:=0 to high(src) do dst[i]:=src[i];
    dst:=Copy(src);
end;
// -----------------------------------------------------------------------
procedure add_vect( var y1, y2: DVector; k: extended); // y1 += y2*k
var i: integer;
begin
	if Length(y1)<>Length(y2) then SetLength(y2, Length(y1));
	for i:=0 to high(y1) do y1[i]:=y1[i] + k*y2[i];
end;
// ----------------------------------------------------------------------
procedure sav_vect( var y: DVector; fname: string); //   
var f: text;
	i: integer;
begin
  Assign( f, fname);
  Rewrite( f );
  for i:=0 to High(y) do
  Write( f, N2Str(y[i],14), ' ' );
  WriteLn( f );
  Close( f );
end;
// ----------------------------------------------------------------------
procedure app_vect( var y: DVector; fname: string); //   
var f: text;
	i: integer;
begin
  hOpen( fname, f);
  for i:=0 to High(y) do
  Write( f, N2Str(y[i],16), ' ' );
  WriteLn( f );
  Close( f );
end;
// ----------------------------------------------------------------------
//var
	//d3,d4,d5,d6: extended;
    //r : RungeKutt;
begin
	//System_d( 0.1, 0.2, 0.3, 0.9, 0.5, 0.7, -0.1, 1, d3,d4,d5,d6);
    //r := RungeKutt.Create;
    //r.set56(1./3, 1./2, 2./3, 3./4, 9./25, 1./3);
    //r.set56(1./3, 1./2, 2./3, 3./4, 9./25, 2./7);
{
  :
 [ 0       0       0      0      0      0      0]
 [                                              ]
 [1/3      0       0      0      0      0      0]
 [                                              ]
 [-5/8    9/8      0      0      0      0      0]
 [                                              ]
 [-2/3     1      1/3     0      0      0      0]
 [                                              ]
 [-179    75      25     15                     ]
 [----    ---     --     ---     0      0      0]
 [448     112     56     448                    ]
 [                                              ]
 [                              112             ]
 [7/6     7/10    5/3    -10    ---     0      0]
 [                              15              ]
 [                                              ]
 [                       -9     224             ]
 [1/9     9/25    2/5    --     ---    1/30    0]
 [                       10     225             ]
}
end.



